{-# OPTIONS_GHC  -XTypeSynonymInstances -XFlexibleInstances  #-}

--------------------------------
-------Resource Aware ML--------
--------------------------------
-- File:
--   ATypes.hs
--
-- Author:
--   Klaus Aehlig, Jan Hoffmann (2010)
--
-- Description:


module ATypes where

import RAMLTypes
import MultivariatePol
import Util
import qualified Data.Map as M


------------------------------------------------------
-- Annotated Types
------------------------------------------------------

data AType = AType (Type,MPolynomial CVarId)


instance Show AType where -- it is standard to display types WITHOUT annotations

   show (AType (t,_)) = show t

instance Typed AType where
    typeOf (AType (t,_)) = t


data AFunType = AFunType AType AType


instance Show AFunType where -- it is standard to display types WITHOUT annotations

   show (AFunType t1 t2) = (show t1) ++ " -> " ++ (show t2)


data AContext = AContext [VarId] AType



mPoly (AType (t, MPolynomial p)) = p

------------------------------------------------------
-- Constant coefficient
------------------------------------------------------

class Annotated a where
    q0 :: a -> CVarId


instance Annotated (AType) where
    q0 (AType (t, MPolynomial p)) = p M.! (the (indicesWithDegree 0 t))


instance Annotated (AContext) where
    q0 (AContext _ t) = q0 t



-------------------------------------------------------
-- Objective Function
-------------------------------------------------------

-- To compute the variables of degree k for a given A type:
varsOfDegree k (AType (t,MPolynomial pol) ) =  foldl (find pol) [] (indicesWithDegree k t)
   where find pol acc ind = case M.lookup ind pol of
                              Nothing -> acc
                              Just var -> var:acc


objectivesFromFun :: AFunType -> [Objective]
objectivesFromFun (AFunType tArg tRes) = obj 0 tArg
   where
     obj k atype = case varsOfDegree k atype of
                     []   -> []
                     vars -> (map (\var ->  Objective ((k+1)*(k+1)*(k+1)*(k*1)) var ) vars) ++ (obj (k+1) atype)
