--------------------------------
-------Resource Aware ML--------
--------------------------------
-- File:
--   CallGraph.hs
--
-- Author:
--   Jan Hoffmann (2009)
--
-- Description: 
--   Stuff that deals with the call graph of a program.
--
--   The function callGraph takes a list of function definitions and
--   coumputes the call graph.
--
--   The function getSCCs computes the strongly connected components
--   of the call graph from a list of function definitions..

module CallGraph where

import Data.List
import Data.Graph

import RAMLTypes

-- Following the GHC graph library a graph (in list form) is a list of
-- triples (nodeName, nodeKey, successorKeys).

callGraph funList = [ (f,f,functionCalls e_f) | (f,_,e_f,_) <- funList ]
   where
      functionCalls expr = case expr of
          EApp fName e _ -> [fName] `union` functionCalls e
          ENot e _ -> functionCalls e
          EBinOp _ e1 e2 _ -> (functionCalls e1) `union` (functionCalls e2)
          ELet x e1 e2 _ -> (functionCalls e1) `union` (functionCalls e2)
          ECond e e1 e2 _ -> (functionCalls e) `union` (functionCalls e1) `union` (functionCalls e2)
          ECons e1 e2 _ ->  (functionCalls e1) `union` (functionCalls e2)
          EMatchL _ e e1 x1 x2 e2 _ -> (functionCalls e) `union` (functionCalls e1) `union` (functionCalls e2)
          ETuple es _ -> foldl union [] [functionCalls e | e <- es]
          EMatchT e xs e' _ -> (functionCalls e) `union` (functionCalls e')
          _ -> []



getSCCs funList = map flattenSCC (stronglyConnComp (callGraph funList) )

